class DataHandler:
    @staticmethod
    def set_template(
        serverless,
        template_name,
        container_image,
        model_name,
        max_model_length,
        container_disk,
        volume_disk,
        volume_mount_path="",
        ports="",
        container_cmd="",
        readme="",
    ):
        data = {
            "query": f"""mutation {{
                saveTemplate(input: {{
                    containerDiskInGb: {container_disk},
                    dockerArgs: "{container_cmd}",
                    env: [
                        {{ key: "MODEL_NAME", value: "{model_name}" }},
                        {{ key: "MAX_MODEL_LENGTH", value: "{max_model_length}" }}
                    ],
                    imageName: "{container_image}",
                    isServerless: {serverless},
                    name: "{template_name}",
                    ports: "{ports}",
                    readme: "{readme}",
                    volumeInGb: {volume_disk},
                    volumeMountPath: "{volume_mount_path}"
                }}) {{
                    containerDiskInGb
                    dockerArgs
                    env {{
                        key
                        value
                    }}
                    id
                    imageName
                    name
                    ports
                    readme
                    volumeInGb
                    volumeMountPath
                }}
            }}"""
        }
        return data

    @staticmethod
    def set_endpoint(
        gpu_ids,
        idle_timeout,
        name,
        scaler_type,
        scaler_value,
        template_id,
        workers_max,
        workers_min,
        locations="",
        network_volume_id="",
    ):
        data = {
            "query": f"""mutation {{
            saveEndpoint(input: {{
                gpuIds: "{gpu_ids}",
                idleTimeout: {idle_timeout},
                locations: "{locations}",
                name: "{name}",
                networkVolumeId: "{network_volume_id}",
                scalerType: "{scaler_type}",
                scalerValue: {scaler_value},
                templateId: "{template_id}",
                workersMax: {workers_max},
                workersMin: {workers_min}
            }}) {{
                gpuIds
                id
                idleTimeout
                locations
                name
                scalerType
                scalerValue
                templateId
                workersMax
                workersMin
            }}
        }}"""
        }

        return data

    @staticmethod
    def check_endpoint():
        data = {
            "query": """
            query Endpoints {
                myself {
                    endpoints {
                        gpuIds
                        id
                        idleTimeout
                        locations
                        name
                        networkVolumeId
                        pods {
                            desiredStatus
                            id
                            lastStatusChange
                        }
                        scalerType
                        scalerValue
                        templateId
                        workersMax
                        workersMin
                    }
                    serverlessDiscount {
                        discountFactor
                        type
                        expirationDate
                    }
                }
            }
            """
        }

        return data



from huggingface_hub import HfApi, snapshot_download, create_repo


class HuggingFaceHandler:
    def __init__(
        self,
        token,
        model_id,
        local_dir,
        repo_id,
        max_workers=4,
        repo_type="model",
        private_repo=False,
    ):
        self.api = HfApi(token=token)
        self.model_id = model_id
        self.local_dir = local_dir
        self.max_workers = max_workers
        self.repo_id = repo_id
        self.repo_type = repo_type
        self.private_repo = private_repo

    def download_model_snapshot(self):
        snapshot_download(
            self.model_id, local_dir=self.local_dir, max_workers=self.max_workers
        )

    def upload_folder(self):
        self.api.upload_folder(
            folder_path=self.local_dir, repo_id=self.repo_id, repo_type=self.repo_type
        )

    def create_repository(self, repo_name):
        create_repo(
            repo_name,
            token=self.api.token,
            private=self.private_repo,
            repo_type=self.repo_type,
        )


# Example usage:
# hf_handler = HuggingFaceHandler(
#     token="your_token_here",
#     model_id="mistralai/Mistral-7B-Instruct-v0.2",
#     local_dir="tmp/hf_models/mistral-7b-instruct-v0.2",
#     repo_id="your_repo_id",
#     repo_type="model",
#     private_repo=True
# )
# hf_handler.create_repository("zeroshot/test")
# hf_handler.download_model_snapshot()
# hf_handler.upload_folder()



from pydantic import BaseModel, root_validator, constr, conint, confloat


class LLMSettings(BaseModel):
    MODEL_NAME: str
    MODEL_REVISION: str = None
    MAX_MODEL_LENGTH: int = 512
    BASE_PATH: str = "/runpod-volume"
    LOAD_FORMAT: str = "auto"
    HF_TOKEN: str = None
    QUANTIZATION: str = None
    TRUST_REMOTE_CODE: int = 0
    SEED: int = 0
    KV_CACHE_DTYPE: str = "auto"
    DTYPE: constr(regex=r"auto|half|float16|bfloat16|float|float32") = "auto"

    class Config:
        allow_population_by_field_name = True


class TokenizerSettings(BaseModel):
    TOKENIZER_NAME: str = None
    TOKENIZER_REVISION: str = None
    CUSTOM_CHAT_TEMPLATE: str = None

    class Config:
        allow_population_by_field_name = True


class SystemGPUAndTensorParallelismSettings(BaseModel):
    GPU_MEMORY_UTILIZATION: confloat(ge=0, le=1) = 0.95
    MAX_PARALLEL_LOADING_WORKERS: int = None
    BLOCK_SIZE: conint(gt=0) = 16
    SWAP_SPACE: conint(gt=0) = 4
    ENFORCE_EAGER: int = 0
    MAX_CONTEXT_LEN_TO_CAPTURE: conint(gt=0) = 8192
    DISABLE_CUSTOM_ALL_REDUCE: int = 0

    class Config:
        allow_population_by_field_name = True


class StreamingBatchSizeSettings(BaseModel):
    DEFAULT_BATCH_SIZE: conint(gt=0) = 50
    DEFAULT_MIN_BATCH_SIZE: conint(gt=0) = 1
    DEFAULT_BATCH_SIZE_GROWTH_FACTOR: confloat(gt=0) = 3

    class Config:
        allow_population_by_field_name = True


class OpenAISettings(BaseModel):
    RAW_OPENAI_OUTPUT: int = 1
    OPENAI_SERVED_MODEL_NAME_OVERRIDE: str = None
    OPENAI_RESPONSE_ROLE: constr(regex=r"assistant|other_role") = "assistant"

    class Config:
        allow_population_by_field_name = True


class ServerlessSettings(BaseModel):
    MAX_CONCURRENCY: conint(gt=0) = 300
    DISABLE_LOG_STATS: int = 1
    DISABLE_LOG_REQUESTS: int = 1

    class Config:
        allow_population_by_field_name = True


class AllSettings(BaseModel):
    LLM_Settings: LLMSettings
    Tokenizer_Settings: TokenizerSettings
    System_GPU_and_Tensor_Parallelism_Settings: SystemGPUAndTensorParallelismSettings
    Streaming_Batch_Size_Settings: StreamingBatchSizeSettings
    OpenAI_Settings: OpenAISettings
    Serverless_Settings: ServerlessSettings

    class Config:
        allow_population_by_field_name = True

    @root_validator
    def check_settings(cls, values):
        if (
            values["LLM_Settings"].LOAD_FORMAT == "auto"
            and values["LLM_Settings"].MODEL_REVISION is not None
        ):
            raise ValueError(
                "MODEL_REVISION cannot be specified when LOAD_FORMAT is set to 'auto'"
            )
        return values



class ApiKeyError(Exception):
    """
    Raised when API key is not provided.
    """

    def __init__(self):
        super().__init__(
            "API key not provided. Please set the RUNPOD_API_KEY environment variable for proper API security."
        )



import os
import requests

from .data import DataHandler
from ..utils.errata import ApiKeyError


class RunpodHandler(DataHandler):
    def __init__(self, api_key=None, data=None):
        self.api_key = api_key or os.getenv("RUNPOD_API_KEY")
        if not self.api_key:
            raise ApiKeyError()
        self.url = f"https://api.runpod.io/graphql?api_key={self.api_key}"
        self.data = data
        self.method = "POST"
        self.headers = {"content-type": "application/json"}

    def __call__(self):
        response = requests.request(
            method=self.method, url=self.url, headers=self.headers, json=self.data
        )

        if response.status_code != 200:
            print("Error:", response.text)

        return response



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    LONG_DESCRIPTION = "\n" + fh.read()

NAME = "superlaser"
VERSION = "0.0.6"
AUTHOR = "InquestGeronimo"
EMAIL = "rcostanl@gmail.com"
LD_CONTENT_TYPE = "text/markdown"
DESCRIPTION = (
    "An MLOps library for LLM deployment w/ the vLLM engine on RunPod's infra."
)
LICENSE = "Apache-2.0 license"
PACKAGES = find_packages()
DEPENDENCIES = ["huggingface_hub>=0.21.1", "openai>=1.13.3"]
KEYWORDS = [
    "inference",
    "server",
    "LLM",
    "NLP",
    "MLOps",
    "deployment",
    "vllm",
    "runpod",
    "cicd",
]
CLASSIFIERS = [
    "Development Status :: 1 - Planning",
    "Intended Audience :: Developers",
    "Programming Language :: Python :: 3",
    "Operating System :: Unix",
]

setup(
    name=NAME,
    version=VERSION,
    license=LICENSE,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    long_description_content_type=LD_CONTENT_TYPE,
    long_description=LONG_DESCRIPTION,
    packages=PACKAGES,
    install_requires=DEPENDENCIES,
    keywords=KEYWORDS,
    classifiers=CLASSIFIERS,
)
